<?php
/*


Plugin Name: Goalcoin Payments for WooCommerce
Plugin URI: http://www.goalco.in/
Description: Goalcoin Payments for WooCommerce plugin allows you to accept payments in goalcoins for physical and digital products at your WooCommerce-powered online store.
Version: 1.0
Author: Goalcoin-OtZI
License: GNU General Public License 2.0 (GPL) http://www.gnu.org/licenses/gpl.html

*/


// Include everything
include (dirname(__FILE__) . '/wcgoal-include-all.php');

//---------------------------------------------------------------------------
// Add hooks and filters

// create custom plugin settings menu
add_action( 'admin_menu',                   'WCGOAL_create_menu' );

register_activation_hook(__FILE__,          'WCGOAL_activate');
register_deactivation_hook(__FILE__,        'WCGOAL_deactivate');
register_uninstall_hook(__FILE__,           'WCGOAL_uninstall');

add_filter ('cron_schedules',               'WCGOAL__add_custom_scheduled_intervals');
add_action ('WCGOAL_cron_action',             'WCGOAL_cron_job_worker');     // Multiple functions can be attached to 'WCGOAL_cron_action' action

WCGOAL_set_lang_file();
//---------------------------------------------------------------------------

//===========================================================================
// activating the default values
function WCGOAL_activate()
{
    global  $g_wcgoal__config_defaults;

    $wcgoal_default_options = $g_wcgoal__config_defaults;

    // This will overwrite default options with already existing options but leave new options (in case of upgrading to new version) untouched.
    $wcgoal_settings = WCGOAL__get_settings ();

    foreach ($wcgoal_settings as $key=>$value)
    	$wcgoal_default_options[$key] = $value;

    update_option (WCGOAL_SETTINGS_NAME, $wcgoal_default_options);

    // Re-get new settings.
    $wcgoal_settings = WCGOAL__get_settings ();

    // Create necessary database tables if not already exists...
    WCGOAL__create_database_tables ($wcgoal_settings);

    //----------------------------------
    // Setup cron jobs

    if ($wcgoal_settings['enable_soft_cron_job'] && !wp_next_scheduled('WCGOAL_cron_action'))
    {
    	$cron_job_schedule_name = strpos($_SERVER['HTTP_HOST'], 'ttt.com')===FALSE ? $wcgoal_settings['soft_cron_job_schedule_name'] : 'seconds_30';
    	wp_schedule_event(time(), $cron_job_schedule_name, 'WCGOAL_cron_action');
    }
    //----------------------------------

}
//---------------------------------------------------------------------------
// Cron Subfunctions
function WCGOAL__add_custom_scheduled_intervals ($schedules)
{
	$schedules['seconds_30']     = array('interval'=>30,     'display'=>__('Once every 30 seconds'));     // For testing only.
	$schedules['minutes_1']      = array('interval'=>1*60,   'display'=>__('Once every 1 minute'));
	$schedules['minutes_2.5']    = array('interval'=>2.5*60, 'display'=>__('Once every 2.5 minutes'));
	$schedules['minutes_5']      = array('interval'=>5*60,   'display'=>__('Once every 5 minutes'));

	return $schedules;
}
//---------------------------------------------------------------------------
//===========================================================================

//===========================================================================
// deactivating
function WCGOAL_deactivate ()
{
    // Do deactivation cleanup. Do not delete previous settings in case user will reactivate plugin again...

   //----------------------------------
   // Clear cron jobs
   wp_clear_scheduled_hook ('WCGOAL_cron_action');
   //----------------------------------
}
//===========================================================================

//===========================================================================
// uninstalling
function WCGOAL_uninstall ()
{
    $wcgoal_settings = WCGOAL__get_settings();

    if ($wcgoal_settings['delete_db_tables_on_uninstall'])
    {
        // delete all settings.
        delete_option(WCGOAL_SETTINGS_NAME);

        // delete all DB tables and data.
        WCGOAL__delete_database_tables ();
    }
}
//===========================================================================

//===========================================================================
function WCGOAL_create_menu()
{

    // create new top-level menu
    // http://www.fileformat.info/info/unicode/char/e3f/index.htm
    add_menu_page (
        __('Woo Goalcoin', WCGOAL_I18N_DOMAIN),                    // Page title
        __('Goalcoin', WCGOAL_I18N_DOMAIN),                        // Menu Title - lower corner of admin menu
        'administrator',                                        // Capability
        'wcgoal-settings',                                        // Handle - First submenu's handle must be equal to parent's handle to avoid duplicate menu entry.
        'WCGOAL__render_general_settings_page',                   // Function
        plugins_url('/images/goalcoin_16x.png', __FILE__)                // Icon URL
        );

    add_submenu_page (
        'wcgoal-settings',                                        // Parent
        __("WooCommerce Goalcoin Payments Gateway", WCGOAL_I18N_DOMAIN),                   // Page title
        __("General Settings", WCGOAL_I18N_DOMAIN),               // Menu Title
        'administrator',                                        // Capability
        'wcgoal-settings',                                        // Handle - First submenu's handle must be equal to parent's handle to avoid duplicate menu entry.
        'WCGOAL__render_general_settings_page'                    // Function
        );
}
//===========================================================================

//===========================================================================
// load language files
function WCGOAL_set_lang_file()
{
    # set the language file
    $currentLocale = get_locale();
    if(!empty($currentLocale))
    {
        $moFile = dirname(__FILE__) . "/lang/" . $currentLocale . ".mo";
        if (@file_exists($moFile) && is_readable($moFile))
        {
            load_textdomain(WCGOAL_I18N_DOMAIN, $moFile);
        }

    }
}
//===========================================================================

